@@ -1,18 +1,18 @@
 use std::borrow::Cow;
-use std::io::{self, BufRead, BufReader, BufWriter, Read, Write};
+use std::io::{self, BufRead, BufReader, Read, Write};
 
 use encoding_rs::Encoding;
 use encoding_rs_io::DecodeReaderBytesBuilder;
 use mime::Mime;
-use reqwest::blocking::{Request, Response};
+use reqwest::blocking::{Body, Request, Response};
 use reqwest::cookie::CookieStore;
 use reqwest::header::{
     HeaderMap, HeaderName, HeaderValue, ACCEPT, CONTENT_LENGTH, CONTENT_TYPE, COOKIE, HOST,
 };
 use reqwest::Version;
-use termcolor::WriteColor;
 use url::Url;
 
+use crate::decoder::{decompress, get_compression_type};
 use crate::{
     buffer::Buffer,
     cli::{Pretty, Theme},
@@ -50,19 +50,57 @@ impl<'a, T: Read> BinaryGuard<'a, T> {
         }
     }
 
-    fn read_line(&mut self) -> io::Result<Option<&[u8]>> {
+    /// Return at least one complete line.
+    ///
+    /// Compared to returning exactly one line, this gives you more information
+    /// about when data comes in. It's better to flush after each `read_lines`
+    /// call than to flush after each individual line.
+    ///
+    /// We only work with complete lines to accommodate the syntax highlighting
+    /// and the binary data (null byte) detection. HTTPie processes exactly
+    /// one line at a time.
+    ///
+    /// We work off the assumption that if the response contains a null byte
+    /// then none of it should be shown, and therefore the earlier we detect
+    /// the null byte, the better. This basically matches the non-streaming
+    /// behavior. But if it takes a while for the first null byte to show up
+    /// then it's unpredictable when the plain text output is cut off by the
+    /// binary suppressor. HTTPie is more consistent in this regard.
+    fn read_lines(&mut self) -> io::Result<Option<&[u8]>> {
         self.buffer.clear();
-        self.reader.read_until(b'\n', &mut self.buffer)?;
-        if self.buffer.is_empty() {
-            return Ok(None);
-        }
-        if self.checked && self.buffer.contains(&b'\0') {
-            Err(io::Error::new(
-                io::ErrorKind::InvalidData,
-                "Found binary data",
-            ))
-        } else {
-            Ok(Some(&self.buffer))
+        loop {
+            let buf = match self.reader.fill_buf() {
+                Ok(buf) => buf,
+                Err(e) if e.kind() == io::ErrorKind::Interrupted => continue,
+                Err(e) => return Err(e),
+            };
+            if self.checked && buf.contains(&b'\0') {
+                return Err(io::Error::new(
+                    io::ErrorKind::InvalidData,
+                    "Found binary data",
+                ));
+            } else if buf.is_empty() {
+                if self.buffer.is_empty() {
+                    return Ok(None);
+                } else {
+                    return Ok(Some(&self.buffer));
+                }
+            } else if let Some(ind) = memchr::memrchr(b'\n', buf) {
+                // Potential optimization: return a slice of buf instead of copying.
+                // (We'd have to delay the call to .consume() until the next call.)
+                // (There is a weird borrow checker problem.)
+                self.buffer.extend_from_slice(&buf[..=ind]);
+                self.reader.consume(ind + 1);
+                return Ok(Some(&self.buffer));
+            } else {
+                self.buffer.extend_from_slice(buf);
+                let n = buf.len(); // borrow checker
+                self.reader.consume(n);
+                // It would be nice to return early if self.buffer is growing very large
+                // or if it's been a long time since the last read. But especially the
+                // second is hard to implement, and we'd want to pair this with flushing
+                // the output buffer. (HTTPie does nothing of this kind.)
+            }
         }
     }
 }
@@ -83,7 +121,7 @@ impl Printer {
         Printer {
             indent_json: pretty.format(),
             sort_headers: pretty.format(),
-            color: pretty.color() && (cfg!(test) || buffer.supports_color()),
+            color: pretty.color(),
             stream,
             theme,
             buffer,
@@ -95,13 +133,6 @@ impl Printer {
     }
 
     fn print_colorized_text(&mut self, text: &str, syntax: &'static str) -> io::Result<()> {
-        // This could perhaps be optimized
-        // syntect processes the whole buffer at once, doing it line by line might
-        // let us start printing earlier (but can decrease quality since regexes
-        // can't look ahead)
-        // A buffered writer could improve performance, but we'd have to use a
-        // BufferedStandardStream instead of a StandardStream, which is slightly tricky
-        // (wrapping a BufWriter around a Buffer wouldn't preserve syntax coloring)
         self.get_highlighter(syntax).highlight(text)
     }
 
@@ -134,9 +165,7 @@ impl Printer {
             let text = String::from_utf8_lossy(&buf);
             self.print_colorized_text(&text, "json")
         } else {
-            let mut out = BufWriter::new(&mut self.buffer);
-            get_json_formatter().format_buf(text.as_bytes(), &mut out)?;
-            out.flush()
+            get_json_formatter().format_buf(text.as_bytes(), &mut self.buffer)
         }
     }
 
@@ -159,11 +188,12 @@ impl Printer {
 
     fn print_stream(&mut self, reader: &mut impl Read) -> io::Result<()> {
         if !self.buffer.is_terminal() {
-            return copy_largebuf(reader, &mut self.buffer);
+            return copy_largebuf(reader, &mut self.buffer, true);
         }
         let mut guard = BinaryGuard::new(reader, true);
-        while let Some(line) = guard.read_line()? {
-            self.buffer.print(line)?;
+        while let Some(lines) = guard.read_lines()? {
+            self.buffer.write_all(lines)?;
+            self.buffer.flush()?;
         }
         Ok(())
     }
@@ -175,8 +205,11 @@ impl Printer {
     ) -> io::Result<()> {
         let mut guard = BinaryGuard::new(stream, self.buffer.is_terminal());
         let mut highlighter = self.get_highlighter(syntax);
-        while let Some(line) = guard.read_line()? {
-            highlighter.highlight_bytes(line)?;
+        while let Some(lines) = guard.read_lines()? {
+            for line in lines.split_inclusive(|&b| b == b'\n') {
+                highlighter.highlight_bytes(line)?;
+            }
+            highlighter.flush()?;
         }
         Ok(())
     }
@@ -202,20 +235,35 @@ impl Printer {
             let mut formatter = get_json_formatter();
             let mut highlighter = self.get_highlighter("json");
             let mut buf = Vec::new();
-            while let Some(line) = guard.read_line()? {
-                formatter.format_buf(line, &mut buf)?;
-                highlighter.highlight_bytes(&buf)?;
+            while let Some(lines) = guard.read_lines()? {
+                formatter.format_buf(lines, &mut buf)?;
+                for line in buf.split_inclusive(|&b| b == b'\n') {
+                    highlighter.highlight_bytes(line)?;
+                }
+                highlighter.flush()?;
                 buf.clear();
             }
             Ok(())
         } else {
             let mut formatter = get_json_formatter();
             if !self.buffer.is_terminal() {
-                return formatter.format_stream_unbuffered(stream, &mut self.buffer);
+                let mut buf = vec![0; BUFFER_SIZE];
+                loop {
+                    match stream.read(&mut buf) {
+                        Ok(0) => return Ok(()),
+                        Ok(n) => {
+                            formatter.format_buf(&buf[0..n], &mut self.buffer)?;
+                            self.buffer.flush()?;
+                        }
+                        Err(e) if e.kind() == io::ErrorKind::Interrupted => continue,
+                        Err(e) => return Err(e),
+                    }
+                }
             }
             let mut guard = BinaryGuard::new(stream, true);
-            while let Some(line) = guard.read_line()? {
-                formatter.format_buf(line, &mut self.buffer)?;
+            while let Some(lines) = guard.read_lines()? {
+                formatter.format_buf(lines, &mut self.buffer)?;
+                self.buffer.flush()?;
             }
             Ok(())
         }
@@ -287,6 +335,7 @@ impl Printer {
 
     pub fn print_separator(&mut self) -> io::Result<()> {
         self.buffer.print("\n")?;
+        self.buffer.flush()?;
         Ok(())
     }
 
@@ -311,7 +360,7 @@ impl Printer {
         // See https://github.com/seanmonstar/reqwest/issues/1030
         // reqwest and hyper add certain headers, but only in the process of
         // sending the request, which we haven't done yet
-        if let Some(body) = request.body().and_then(|body| body.as_bytes()) {
+        if let Some(body) = request.body().and_then(Body::as_bytes) {
             // Added at https://github.com/seanmonstar/reqwest/blob/e56bd160ba/src/blocking/request.rs#L132
             headers
                 .entry(CONTENT_LENGTH)
@@ -338,6 +387,7 @@ impl Printer {
 
         self.print_headers(&(request_line + &headers))?;
         self.buffer.print("\n\n")?;
+        self.buffer.flush()?;
         Ok(())
     }
 
@@ -351,6 +401,7 @@ impl Printer {
 
         self.print_headers(&(status_line + &headers))?;
         self.buffer.print("\n\n")?;
+        self.buffer.flush()?;
         Ok(())
     }
 
@@ -366,21 +417,23 @@ impl Printer {
             }
             // Breathing room between request and response
             self.buffer.print("\n")?;
+            self.buffer.flush()?;
         }
         Ok(())
     }
 
     pub fn print_response_body(
         &mut self,
-        mut response: Response,
+        response: &mut Response,
         encoding: Option<&'static Encoding>,
         mime: Option<&str>,
     ) -> anyhow::Result<()> {
-        let url = response.url().to_owned();
-        let content_type = mime
-            .map(ContentType::from)
-            .unwrap_or_else(|| get_content_type(response.headers()));
-        let encoding = encoding.or_else(|| get_charset(&response));
+        let url = response.url().clone();
+        let content_type =
+            mime.map_or_else(|| get_content_type(response.headers()), ContentType::from);
+        let encoding = encoding.or_else(|| get_charset(response));
+        let compression_type = get_compression_type(response.headers());
+        let mut body = decompress(response, compression_type);
 
         if !self.buffer.is_terminal() {
             if (self.color || self.indent_json) && content_type.is_text() {
@@ -399,24 +452,25 @@ impl Printer {
                 if self.stream {
                     self.print_body_stream(
                         content_type,
-                        &mut decode_stream(&mut response, encoding, &url)?,
+                        &mut decode_stream(&mut body, encoding, &url)?,
                     )?;
                 } else {
-                    let bytes = response.bytes()?;
-                    let text = decode_blob_unconditional(&bytes, encoding, &url);
+                    let mut buf = Vec::new();
+                    body.read_to_end(&mut buf)?;
+                    let text = decode_blob_unconditional(&buf, encoding, &url);
                     self.print_body_text(content_type, &text)?;
                 }
             } else if self.stream {
-                copy_largebuf(&mut response, &mut self.buffer)?;
+                copy_largebuf(&mut body, &mut self.buffer, true)?;
             } else {
-                let body = response.bytes()?;
-                self.buffer.print(&body)?;
+                let mut buf = Vec::new();
+                body.read_to_end(&mut buf)?;
+                self.buffer.print(&buf)?;
             }
         } else if self.stream {
-            match self.print_body_stream(
-                content_type,
-                &mut decode_stream(&mut response, encoding, &url)?,
-            ) {
+            match self
+                .print_body_stream(content_type, &mut decode_stream(&mut body, encoding, &url)?)
+            {
                 Ok(_) => {
                     self.buffer.print("\n")?;
                 }
@@ -426,23 +480,25 @@ impl Printer {
                 Err(err) => return Err(err.into()),
             }
         } else {
-            // Note that .decode() behaves like String::from_utf8_lossy()
-            let bytes = response.bytes()?;
-            let text = match decode_blob(&bytes, encoding, &url) {
+            let mut buf = Vec::new();
+            body.read_to_end(&mut buf)?;
+            match decode_blob(&buf, encoding, &url) {
                 None => {
                     self.buffer.print(BINARY_SUPPRESSOR)?;
                     return Ok(());
                 }
-                Some(text) => text,
+                Some(text) => {
+                    self.print_body_text(content_type, &text)?;
+                    self.buffer.print("\n")?;
+                }
             };
-            self.print_body_text(content_type, &text)?;
-            self.buffer.print("\n")?;
         }
+        self.buffer.flush()?;
         Ok(())
     }
 }
 
-pub enum ContentType {
+enum ContentType {
     Json,
     Html,
     Xml,
@@ -455,7 +511,7 @@ pub enum ContentType {
 }
 
 impl ContentType {
-    pub fn is_text(&self) -> bool {
+    fn is_text(&self) -> bool {
         !matches!(
             self,
             ContentType::Unknown | ContentType::UrlencodedForm | ContentType::Multipart
@@ -491,15 +547,14 @@ impl From<&str> for ContentType {
     }
 }
 
-pub fn get_content_type(headers: &HeaderMap) -> ContentType {
+fn get_content_type(headers: &HeaderMap) -> ContentType {
     headers
         .get(CONTENT_TYPE)
         .and_then(|value| value.to_str().ok())
-        .map(ContentType::from)
-        .unwrap_or(ContentType::Unknown)
+        .map_or(ContentType::Unknown, ContentType::from)
 }
 
-pub fn valid_json(text: &str) -> bool {
+fn valid_json(text: &str) -> bool {
     serde_json::from_str::<serde::de::IgnoredAny>(text).is_ok()
 }
 
@@ -548,7 +603,7 @@ fn decode_blob_unconditional<'a>(
 /// output is valid UTF-8, but a differently configured DecodeReaderBytes can
 /// produce invalid UTF-8.
 fn decode_stream<'a>(
-    response: &'a mut Response,
+    stream: &'a mut impl Read,
     encoding: Option<&'static Encoding>,
     url: &Url,
 ) -> io::Result<impl Read + 'a> {
@@ -558,7 +613,7 @@ fn decode_stream<'a>(
     // large enough for a best-effort attempt.
     // (16 is otherwise used because 0 seems dangerous, but it shouldn't matter.)
     let capacity = if encoding.is_some() { 16 } else { 16 * 1024 };
-    let mut reader = BufReader::with_capacity(capacity, response);
+    let mut reader = BufReader::with_capacity(capacity, stream);
     let encoding = match encoding {
         Some(encoding) => encoding,
         None => {
@@ -640,12 +695,10 @@ mod tests {
     use super::*;
     use crate::utils::random_string;
     use crate::{buffer::Buffer, cli::Cli, vec_of_strings};
-    use assert_matches::assert_matches;
 
     fn run_cmd(args: impl IntoIterator<Item = String>, is_stdout_tty: bool) -> Printer {
-        let args = Cli::from_iter_safe(args).unwrap();
-        let buffer =
-            Buffer::new(args.download, args.output.as_deref(), is_stdout_tty, None).unwrap();
+        let args = Cli::try_parse_from(args).unwrap();
+        let buffer = Buffer::new(args.download, args.output.as_deref(), is_stdout_tty).unwrap();
         let pretty = args.pretty.unwrap_or_else(|| buffer.guess_pretty());
         Printer::new(pretty, args.style, false, buffer)
     }
@@ -661,14 +714,14 @@ mod tests {
     fn terminal_mode() {
         let p = run_cmd(vec_of_strings!["xh", "httpbin.org/get"], true);
         assert_eq!(p.color, true);
-        assert_matches!(p.buffer, Buffer::Stdout(..));
+        assert!(p.buffer.is_stdout());
     }
 
     #[test]
     fn redirect_mode() {
         let p = run_cmd(vec_of_strings!["xh", "httpbin.org/get"], false);
         assert_eq!(p.color, false);
-        assert_matches!(p.buffer, Buffer::Redirect(..));
+        assert!(p.buffer.is_redirect());
     }
 
     #[test]
@@ -676,7 +729,7 @@ mod tests {
         let output = temp_path();
         let p = run_cmd(vec_of_strings!["xh", "httpbin.org/get", "-o", output], true);
         assert_eq!(p.color, false);
-        assert_matches!(p.buffer, Buffer::File(_));
+        assert!(p.buffer.is_file());
     }
 
     #[test]
@@ -687,21 +740,21 @@ mod tests {
             false,
         );
         assert_eq!(p.color, false);
-        assert_matches!(p.buffer, Buffer::File(_));
+        assert!(p.buffer.is_file());
     }
 
     #[test]
     fn terminal_mode_download() {
         let p = run_cmd(vec_of_strings!["xh", "httpbin.org/get", "-d"], true);
         assert_eq!(p.color, true);
-        assert_matches!(p.buffer, Buffer::Stderr(..));
+        assert!(p.buffer.is_stderr());
     }
 
     #[test]
     fn redirect_mode_download() {
         let p = run_cmd(vec_of_strings!["xh", "httpbin.org/get", "-d"], false);
         assert_eq!(p.color, true);
-        assert_matches!(p.buffer, Buffer::Stderr(..));
+        assert!(p.buffer.is_stderr());
     }
 
     #[test]
@@ -712,7 +765,7 @@ mod tests {
             true,
         );
         assert_eq!(p.color, true);
-        assert_matches!(p.buffer, Buffer::Stderr(..));
+        assert!(p.buffer.is_stderr());
     }
 
     #[test]
@@ -723,7 +776,7 @@ mod tests {
             false,
         );
         assert_eq!(p.color, true);
-        assert_matches!(p.buffer, Buffer::Stderr(..));
+        assert!(p.buffer.is_stderr());
     }
 
     #[test]
@@ -734,7 +787,7 @@ mod tests {
             theme: Theme::auto,
             sort_headers: false,
             stream: false,
-            buffer: Buffer::new(false, None, false, Some(Pretty::none)).unwrap(),
+            buffer: Buffer::new(false, None, false).unwrap(),
         };
 
         let mut headers = HeaderMap::new();
